import * as log from './log'
import * as Mqtt from 'mqtt'
import { post } from 'request'


const BROKER : string = 'mqtt://localhost'
const INFLUXDB : string = 'http://172.16.3.15:8086/write?db=smarthome'
const VERBOSE : boolean = true



const TOPIC_PARSERS : { [key : string] : (msg: string) => string } =     {
    'IoT/espThermometer2/measurement': (msg : string) => {
        let jsonData = JSON.parse(msg)
        return `temperature,location=${jsonData.location} value=${jsonData.temperature}\nbattery,location=${jsonData.location},device=thermometer value=${jsonData.battery}`
    }
}




let mqttClient = Mqtt.connect(BROKER)
mqttClient.on('offline', () => { log.warn("MQTT client is offline") })
mqttClient.on('reconnect', () => { log.warn("MQTT client is reconnecting") })
mqttClient.on('close', () => { log.warn("MQTT connection closed") })

mqttClient.on('connect', () => {
    log.info("MQTT broker connected")
    for (let topic in TOPIC_PARSERS) {
        mqttClient.subscribe(topic)
        log.info(`Subscribed to ${topic}`)
    }
})

mqttClient.on('message', (topic : string, messageBuf : Buffer) => {
    let message = messageBuf.toString('UTF-8')
    if (VERBOSE) {
        log.info(`Message received: topic ${topic}, payload ${message}`)
    }
    if (topic in TOPIC_PARSERS) {
        try {
            let lineProt = TOPIC_PARSERS[topic](message)
            log.info(`parsed message: ${lineProt}`)

            post({
                url: INFLUXDB,
                body: lineProt
            }, (err, res, body) => {
                if (err) {
                    log.error(`Error when sending data to database: ${err}`)
                } else {
                    log.info(`Response from database: ${res}, ${body}`)
                }
            })

        } catch (e) {
            log.error(`Error ${e} when parsing ${topic}, ${message}`)
        }
    }
})





log.info("MqttToInflux started")
